-- *******************************************************
--   KOHA  MARC 21 STANDARD DEFAULT AUTHORITY FRAMEWORKS
--
--        Revised to Update No. 21 (September 2015)
-- *******************************************************

-- ******************************************************
-- KOHA DEFAULT MARC 21 AUTHORITY TYPE.
-- ******************************************************

-- This authority type includes all fields and subfieds used by any
-- authority type.

INSERT INTO auth_types (authtypecode, authtypetext, auth_tag_to_report, summary) VALUES
		('', 'Alle Datensätze', '', '');


-- ******************************************************
-- KOHA SUPPORTED STANDARD MARC 21 AUTHORITY TYPES.
-- ******************************************************

-- These authority types are supported for for guiding the cataloguer to
-- fill authorised values in Koha MARC bibliographic editor.


INSERT INTO auth_types (authtypecode, authtypetext, auth_tag_to_report, summary) VALUES
		('PERSO_NAME', 'Personenname', '100', 'Personenname'),
		('CORPO_NAME', 'Körperschaftsname', '110', 'Körperschaftsname'),
		('MEETI_NAME', 'Kongressname', '111', 'Kongressname'),
		('UNIF_TITLE', 'Einheitstitel', '130', 'Einheitstitel'),
		('CHRON_TERM', 'Zeitschlagwort', '148', 'Zeitschlagwort'),
		('TOPIC_TERM', 'Sachschlagwort', '150', 'Sachschlagwort'),
		('GEOGR_NAME', 'Geografischer Name', '151', 'Geografischer Name'),
		('GENRE/FORM', 'Genre/Formschlagwort', '155', 'Genre/Formschlagwort');


-- ******************************************************
-- KOHA UNSUPPORTED STANDARD MARC 21 AUTHORITY TYPES.
-- ******************************************************

-- These authority types are only supported for guiding the cataloguer to
-- fill authorised values in the Koha MARC bibliographic editor to the
-- extent that they have already been included in a primary authority type,
-- therefore, they have not yet been specified.
--
-- Minimal primary authorities including subdivisions may currently be
-- built by a script which uses the values in bibliographic records but
-- include no tracings and references which are necessarily not present
-- in bibliographic records.


-- INSERT INTO `auth_types` (`authtypecode`, `authtypetext`, `auth_tag_to_report`, `summary`) VALUES
--		('TOPIC_SUBD', 'General Topical Term Subdivision', '180', 'General Topical Term Subdivision'),
--		('GEOGR_SUBD', 'Geographic Subdivision', '181', 'Geographic Subdivision'),
--		('CHRON_SUBD', 'Chronological Subdivision', '182', 'Chronological Subdivision'),
--		('FORM_SUBDI', 'Form Subdivision', '185', 'Form Subdivision');



-- *****************************************************************
-- MARC 21 AUTHORITY FIELDS/SUBFIELDS AND COMMMONLY USED EXTENSIONS
-- *****************************************************************


-- No support for subfield visibility is yet provided by the Koha code
-- matching the column name 'hidden' in the bibliographic framework.
--
-- Needs checking for errors but probably tolerable for use on a production.
-- A server can be upgraded easily from later versions of this file.
--
-- In the absense of more column support for qualifying the relative
-- importance of subfields to the record editor, some modest modification of
-- the default framework is needed setting the not-useful non-Koha holdings
-- subfields to not managed in Koha.

-- MARC fields including letters as part of the field identifier are from RLIN
-- and should be expected to remain along with RLIN $% subfields.  RLIN has
-- been using letters in fields because there are not enough local use number
-- fields which have not already been specified for very large union catalogue
-- networks such as RLIN itself.

-- Fields ending in a or b are temporary placeholders for information from
-- a numeric value until a non-conflicting way to treat the content under the
-- proper original numeric field is adopted.  090 for LC call numbers is much
-- too common and important so 999 is also provided as a temporary place
-- holder until all Koha code for finding control fields has been changed from
-- a numeric test of < 10 to a regular expression match of m/^00/ to prevent
-- mistaken matching of fields with letters such as 09o if they were control
-- fields.

-- Fields and subfields which belong exclusively in a classification
-- framework have been marked '[CLASSIFICATION FRAMEWORK]'.  They are retained
-- or included here becuase some had been included previously and Koha does
-- not yet have a classification framework.


INSERT INTO `auth_tag_structure` (`authtypecode`, `tagfield`, `liblibrarian`, `libopac`, `repeatable`, `mandatory`, `authorised_value`) VALUES
		('', '000', 'Satzkennung', 'Satzkennung', 0, 1, NULL),
		('', '001', 'Kontrollnummer', 'Kontrollnummer', 0, 1, NULL),
		('', '003', 'Kontrollnummer Identifier', 'Kontrollnummer Identifier', 0, 1, NULL),
		('', '005', 'Datum und Zeit der letzten Transaktion', 'Datum und Zeit der letzten Transaktion', 0, 1, NULL),
		('', '008', 'Datenelemente mit fester Länge', 'Datenelemente mit fester Länge', 0, 1, NULL),
		('', '010', 'Kontrollnummer der Library of Congress', 'Kontrollnummer der Library of Congress', 0, 0, NULL),
		('', '014', 'Verknüpfung zum bibliografischen Datensatz für Gesamttitel oder mehrteilige Exemplare', 'Verknüpfung zum bibliografischen Datensatz für Gesamttitel oder mehrteilige Exemplare', 1, 0, NULL),
		('', '016', 'Kontrollnummer der nationalbibliografischen Agentur', 'Kontrollnummer der nationalbibliografischen Agentur', 1, 0, NULL),
		('', '020', 'Internationale Standardbuchnummer', 'Internationale Standardbuchnummer', 1, 0, NULL),
		('', '022', 'Internationale Standardseriennummer', 'Internationale Standardseriennummer', 1, 0, NULL),
		('', '024', 'Anderer Standardidentifier', 'Anderer Standardidentifier', 1, 0, NULL),
		('', '031', 'Informationen zum Musik Incipit', 'Informationen zum Musik Incipit', 1, 0, NULL),
		('', '034', 'Kodierte kartografische mathematische Daten', 'Kodierte kartografische mathematische Daten', 1, 0, NULL),
		('', '035', 'Systemkontrollnummer', 'Systemkontrollnummer', 1, 0, NULL),
		('', '040', 'Katalogisierungsquelle', 'Katalogisierungsquelle', 0, 1, NULL),
		('', '042', 'Authentifizierungscode', 'Authentifizierungscode', 0, 0, NULL),
		('', '043', 'Kode für geografische Gebiete', 'Kode für geografische Gebiete', 0, 0, NULL),
		('', '045', 'Zeitabschnitt der Ansetzung', 'Zeitabschnitt der Ansetzung', 0, 0, NULL),
		('', '046', 'Speziell codiertes Datum', 'Speziell codiertes Datum', 1, 0, NULL),
		('', '050', 'Signatur der Library of Congress', 'Signatur der Library of Congress', 1, 0, NULL),
		('', '052', 'Geografische Klassifikation', 'Geografische Klassifikation', 1, 0, NULL),
		('', '053', 'Notation der LC', 'Notation der LC', 1, 0, NULL),
		('', '055', 'Signatur der Library and Archives Canada', 'Signatur der Library and Archives Canada', 1, 0, NULL),
		('', '058', 'LC CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 'LC CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '060', 'Signatur der National Library of Medicine', 'Signatur der National Library of Medicine', 1, 0, NULL),
		('', '063', 'NLM CLASSIFICATION NUMBER ASSIGNED BY NLM [OBSOLETE, CAN/MARC]', 'NLM CLASSIFICATION NUMBER ASSIGNED BY NLM [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '065', 'Andere Notation', 'Andere Notation', 1, 0, NULL),
		('', '066', 'Angewendete Zeichensätze', 'Angewendete Zeichensätze', 0, 0, NULL),
		('', '068', 'NLM CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 'NLM CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '070', 'Signatur der National Agricultural Library', 'Signatur der National Agricultural Library', 1, 0, NULL),
		('', '072', 'Code der Sacherschließungskategorie', 'Code der Sacherschließungskategorie', 1, 0, NULL),
		('', '073', 'Verwendung der Unterteilung', 'Verwendung der Unterteilung', 0, 0, NULL),
		('', '080', 'Notation nach der Universal Decimal Classification', 'Notation nach der Universal Decimal Classification', 1, 0, NULL),
		('', '082', 'Dewey-Dezimal-Signatur', 'Dewey-Dezimal-Signatur', 1, 0, NULL),
		('', '083', 'Notation nach der Dewey Decimal Classification', 'Notation nach der Dewey Decimal Classification', 1, 0, NULL),
		('', '084', 'CLASSIFICATION SCHEME AND EDITION [CLASSIFICATION FORMAT]', 'CLASSIFICATION SCHEME AND EDITION [CLASSIFICATION FORMAT]', 0, 0, NULL),
		('', '086', 'Signatur für Amtsdruckschriften', 'Signatur für Amtsdruckschriften', 1, 0, NULL),
		('', '087', 'Notation für Amtsdruckschriften', 'Notation für Amtsdruckschriften', 1, 0, NULL),
		('', '088', 'DOCUMENT SHELVING NUMBER (CODOC) [OBSOLETE, CAN/MARC]', 'DOCUMENT SHELVING NUMBER (CODOC) [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 1, 0, NULL),
		('', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
		('', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
		('', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
		('', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
		('', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
		('', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
		('', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
		('', '100', 'Ansetzung - Personenname', 'Ansetzung - Personenname', 0, 0, NULL),
		('', '110', 'Ansetzung - Körperschaftsname', 'Ansetzung - Körperschaftsname', 0, 0, NULL),
		('', '111', 'Ansetzung - Kongressname', 'Ansetzung - Kongressname', 0, 0, NULL),
		('', '130', 'Ansetzung - Einheitstitel', 'Ansetzung - Einheitstitel', 0, 0, NULL),
		('', '140', 'UNIFORM TITLE [OBSOLETE, CAN/MARC]', 'UNIFORM TITLE [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '143', 'COLLECTIVE TITLE [OBSOLETE, CAN/MARC]', 'COLLECTIVE TITLE [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '148', 'Ansetzung - Zeitschlagwort', 'Ansetzung - Zeitschlagwort', 0, 0, NULL),
		('', '150', 'Ansetzung - Zeitschlagwort', 'Ansetzung - Zeitschlagwort', 0, 0, NULL),
		('', '151', 'Ansetzung - Geografischer Name', 'Ansetzung - Geografischer Name', 0, 0, NULL),
		('', '153', 'CLASSIFICATION NUMBER [CLASSIFICATION FORMAT]', 'CLASSIFICATION NUMBER [CLASSIFICATION FORMAT]', 0, 0, NULL),
		('', '154', 'GENERAL EXPLANATORY INDEX TERM [CLASSIFICATION FORMAT]', 'GENERAL EXPLANATORY INDEX TERM [CLASSIFICATION FORMAT]', 0, 0, NULL),
		('', '155', 'Ansetzung - Genre/Form Schlagwort', 'Ansetzung - Genre/Form Schlagwort', 0, 0, NULL),
		('', '162', 'Ansetzung - Bezeichnung der Besetzung', 'Ansetzung - Bezeichnung der Besetzung', 0, 0, NULL),
		('', '180', 'Ansetzung - Allgemeine Unterteilung', 'Ansetzung - Allgemeine Unterteilung', 0, 0, NULL),
		('', '181', 'Ansetzung - Geografische Unterteilung', 'Ansetzung - Geografische Unterteilung', 0, 0, NULL),
		('', '182', 'Ansetzung - Chronologische Unterteilung', 'Ansetzung - Chronologische Unterteilung', 0, 0, NULL),
		('', '185', 'Ansetzung - Unterteilung nach Form', 'Ansetzung - Unterteilung nach Form', 0, 0, NULL),
		('', '253', 'COMPLEX SEE REFERENCE [CLASSIFICATION FORMAT]', 'COMPLEX SEE REFERENCE [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '260', 'Komplexer Siehe-Hinweis - Schlagwort', 'Komplexer Siehe-Hinweis - Schlagwort', 1, 0, NULL),
		('', '336', 'Inhaltstyp', 'Inhaltstyp', 1, 0, NULL),
		('', '348', 'Format der Noten', 'Format der Noten', 1, 0, NULL),
		('', '353', 'COMPLEX SEE ALSO REFERENCE [CLASSIFICATION FORMAT]', 'COMPLEX SEE ALSO REFERENCE [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '360', 'Komplexer Siehe-auch-Hinweis - Schlagwort', 'Komplexer Siehe-auch-Hinweis - Schlagwort', 1, 0, NULL),
		('', '368', 'Andere Eigenschaften der Person oder Körperschaft', 'Andere Eigenschaften der Person oder Körperschaft', 1, 0, NULL),
		('', '370', 'Zugehöriger Ort', 'Zugehöriger Ort', 1, 0, NULL),
		('', '371', 'Adresse', 'Adresse', 1, 0, NULL),
		('', '372', 'Tätigkeitsbereiche', 'Tätigkeitsbereiche', 1, 0, NULL),
		('', '373', 'Zugehörige Gruppe', 'Zugehörige Gruppe', 1, 0, NULL),
		('', '374', 'Beruf oder Beschäftigung', 'Beruf oder Beschäftigung', 1, 0, NULL),
		('', '375', 'Geschlecht', 'Geschlecht', 1, 0, NULL),
		('', '376', 'Informationen zur Familie', 'Informationen zur Familie', 1, 0, NULL),
		('', '377', 'Zugehörige Sprache', 'Zugehörige Sprache', 1, 0, NULL),
		('', '378', 'Vollständigere Namensform', 'Vollständigere Namensform', 0, 0, NULL),
		('', '380', 'Form des Werks', 'Form des Werks', 1, 0, NULL),
		('', '381', 'Sonstige unterscheidende Eigenschaften des Werks oder der Expression', 'Sonstige unterscheidende Eigenschaften des Werks oder der Expression', 1, 0, NULL),
		('', '382', 'Besetzung', 'Besetzung', 1, 0, NULL),
		('', '383', 'Numerische Kennzeichnung eines Musikwerks', 'Numerische Kennzeichnung eines Musikwerks', 1, 0, NULL),
		('', '384', 'Tonart', 'Tonart', 0, 0, NULL),
		('', '385', 'Eigenschaften der Zielgruppe', 'Eigenschaften der Zielgruppe', 1, 0, NULL),
		('', '386', 'Eigenschaften des Urhebers/Mitwirkenden', 'Eigenschaften des Urhebers/Mitwirkenden', 1, 0, NULL),
		('', '388', 'Zeitabschnitt der Erstellung', 'Zeitabschnitt der Erstellung', 1, 0, NULL),
		('', '400', 'Siehe-Verweisung - Personenname', 'Siehe-Verweisung - Personenname', 1, 0, NULL),
		('', '410', 'Siehe-Verweisung - Körperschaftsname', 'Siehe-Verweisung - Körperschaftsname', 1, 0, NULL),
		('', '411', 'Siehe-Verweisung - Kongressname', 'Siehe-Verweisung - Kongressname', 1, 0, NULL),
		('', '430', 'Siehe-Verweisung - Einheitstitel', 'Siehe-Verweisung - Einheitstitel', 1, 0, NULL),
		('', '448', 'Siehe-Verweisung - Zeitschlagwort', 'Siehe-Verweisung - Zeitschlagwort', 1, 0, NULL),
		('', '450', 'Siehe-Verweisung - Sachschlagwort', 'Siehe-Verweisung - Sachschlagwort', 1, 0, NULL),
		('', '451', 'Siehe-Verweisung - Geografischer Name', 'Siehe-Verweisung - Geografischer Name', 1, 0, NULL),
		('', '453', 'INVALID NUMBER TRACING [CLASSIFICATION FORMAT]', 'INVALID NUMBER TRACING [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '455', 'Siehe-Verweisung - Genre/Formschlagwort', 'Siehe-Verweisung - Genre/Formschlagwort', 1, 0, NULL),
		('', '462', 'Siehe-Verweisung - Besetzung', 'Siehe-Verweisung - Besetzung', 1, 0, NULL),
		('', '480', 'Siehe-Verweisung - Allgemeine Unterteilung', 'Siehe-Verweisung - Allgemeine Unterteilung', 1, 0, NULL),
		('', '481', 'Siehe-Verweisung - Geografische Unterteilung', 'Siehe-Verweisung - Geografische Unterteilung', 1, 0, NULL),
		('', '482', 'Siehe-Verweisung - Chronologische Unterteilung', 'Siehe-Verweisung - Chronologische Unterteilung', 1, 0, NULL),
		('', '485', 'Siehe-Verweisung - Unterteilung nach Form', 'Siehe-Verweisung - Unterteilung nach Form', 1, 0, NULL),
		('', '500', 'Siehe-auch-Verweisung - Personenname', 'Siehe-auch-Verweisung - Personenname', 1, 0, NULL),
		('', '510', 'Siehe-auch-Verweisung - Körperschaftsname', 'Siehe-auch-Verweisung - Körperschaftsname', 1, 0, NULL),
		('', '511', 'Siehe-auch-Verweisung - Kongressname', 'Siehe-auch-Verweisung - Kongressname', 1, 0, NULL),
		('', '530', 'Siehe-auch-Verweisung - Einheitstitel', 'Siehe-auch-Verweisung - Einheitstitel', 1, 0, NULL),
		('', '548', 'Siehe-auch-Verweisung - Zeitschlagwort', 'Siehe-auch-Verweisung - Zeitschlagwort', 1, 0, NULL),
		('', '550', 'Siehe-auch-Verweisung - Sachschlagwort', 'Siehe-auch-Verweisung - Sachschlagwort', 1, 0, NULL),
		('', '551', 'Siehe-auch-Verweisung - Geografischer Name', 'Siehe-auch-Verweisung - Geografischer Name', 1, 0, NULL),
		('', '553', 'VALID NUMBER TRACING [CLASSIFICATION FORMAT]', 'VALID NUMBER TRACING [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '555', 'Siehe-auch-Verweisung - Genre/Formschlagwort', 'Siehe-auch-Verweisung - Genre/Formschlagwort', 1, 0, NULL),
		('', '562', 'Siehe-auch-Verweisung - Besetzung', 'Siehe-auch-Verweisung - Besetzung', 1, 0, NULL),
		('', '580', 'Siehe-auch-Verweisung - Allgemeine Unterteilung', 'Siehe-auch-Verweisung - Allgemeine Unterteilung', 1, 0, NULL),
		('', '581', 'Siehe-auch-Verweisung - Geografische Unterteilung', 'Siehe-auch-Verweisung - Geografische Unterteilung', 1, 0, NULL),
		('', '582', 'Siehe-auch-Verweisung - Chronologische Unterteilung', 'Siehe-auch-Verweisung - Chronologische Unterteilung', 1, 0, NULL),
		('', '585', 'Siehe-auch-Verweisung - Unterteilung nach Form', 'Siehe-auch-Verweisung - Unterteilung nach Form', 1, 0, NULL),
		('', '640', 'Erscheinungsjahr und/oder Erscheinungsverlauf von Gesamttiteln', 'Erscheinungsjahr und/oder Erscheinungsverlauf von Gesamttiteln', 1, 0, NULL),
		('', '641', 'Besonderheiten der Zählung von Gesamttiteln', 'Besonderheiten der Zählung von Gesamttiteln', 1, 0, NULL),
		('', '642', 'Beispiel für die Zählung von Gesamttiteln', 'Beispiel für die Zählung von Gesamttiteln', 1, 0, NULL),
		('', '643', 'Erscheinungsort und herausgebende Körperschaft von Gesamttiteln', 'Erscheinungsort und herausgebende Körperschaft von Gesamttiteln', 1, 0, NULL),
		('', '644', 'Analysepraxis von Gesamttiteln', 'Analysepraxis von Gesamttiteln', 1, 0, NULL),
		('', '645', 'Verweisungspraxis von Gesamttiteln', 'Verweisungspraxis von Gesamttiteln', 1, 0, NULL),
		('', '646', 'Klassifizierungspraxis von Gesamttiteln', 'Klassifizierungspraxis von Gesamttiteln', 1, 0, NULL),
		('', '663', 'Komplexe Siehe-auch-Verweisung - Name', 'Komplexe Siehe-auch-Verweisung - Name', 0, 0, NULL),
		('', '664', 'Komplexe Siehe-Verweisung - Name', 'Komplexe Siehe-Verweisung - Name', 0, 0, NULL),
		('', '665', 'Geschichte der Verweisung', 'Geschichte der Verweisung', 0, 0, NULL),
		('', '666', 'Allgemein erläuternde Verweisung - Name', 'Allgemein erläuternde Verweisung - Name', 0, 0, NULL),
		('', '667', 'Interne allgemeine Fußnote', 'Interne allgemeine Fußnote', 1, 0, NULL),
		('', '668', 'CHARACTERS IN NONROMAN ALPHABETS [OBSOLETE]', 'CHARACTERS IN NONROMAN ALPHABETS [OBSOLETE]', 1, 0, NULL),
		('', '670', 'Datenquelle', 'Datenquelle', 1, 0, NULL),
		('', '671', 'NOTE--WORK CATALOGUED [OBSOLETE, CAN/MARC ONLY]', 'NOTE--WORK CATALOGUED [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
		('', '672', 'Titel, der mit der Entität verknüpft ist', 'Titel, der mit der Entität verknüpft ist', 1, 0, NULL),
		('', '673', 'Titel, der nicht mit der Entität verknüpft ist', 'Titel, der nicht mit der Entität verknüpft ist', 1, 0, NULL),
		('', '675', 'Datenquelle ohne Information', 'Datenquelle ohne Information', 0, 0, NULL),
		('', '676', 'NOTE--CATALOGUING RULES (NAMES/TITLES) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--CATALOGUING RULES (NAMES/TITLES) [OBSOLETE, CAN/MARC ONLY]', 0, 0, NULL),
		('', '678', 'Fußnote zu biografischen oder historischen Daten', 'Fußnote zu biografischen oder historischen Daten', 1, 0, NULL),
		('', '680', 'Allgemeine Fußnote', 'Allgemeine Fußnote', 1, 0, NULL),
		('', '681', 'Beispiel für die Schlagwortverweisung', 'Beispiel für die Schlagwortverweisung', 1, 0, NULL),
		('', '682', 'Beispiel für die Schlagwortverweisung', 'Beispiel für die Schlagwortverweisung', 0, 0, NULL),
		('', '683', 'Fußnote zur Anwendungsgeschichte', 'Fußnote zur Anwendungsgeschichte', 1, 0, NULL),
		('', '684', 'AUXILIARY INSTRUCTION NOTE [CLASSIFICATION FORMAT]', 'AUXILIARY INSTRUCTION NOTE [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '685', 'NOTE--SOURCE DATA FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--SOURCE DATA FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
		('', '686', 'NOTE--SOURCE DATA NOT FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--SOURCE DATA NOT FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
		('', '687', 'NOTE--USAGE (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--USAGE (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
		('', '688', 'Fußnote zur Anwendungsgeschichte', 'Fußnote zur Anwendungsgeschichte', 1, 0, NULL),
		('', '700', 'Verknüpfungseintragung für normierte Ansetzungen - Personenname', 'Verknüpfungseintragung für normierte Ansetzungen - Personenname', 1, 0, NULL),
		('', '710', 'Verknüpfungseintragung für normierte Ansetzungen - Körperschaftsname', 'Verknüpfungseintragung für normierte Ansetzungen - Körperschaftsname', 1, 0, NULL),
		('', '711', 'Verknüpfungseintragung für normierte Ansetzungen - Kongressname', 'Verknüpfungseintragung für normierte Ansetzungen - Kongressname', 1, 0, NULL),
		('', '730', 'Verknüpfungseintragung für normierte Ansetzungen - Einheitstitel', 'Verknüpfungseintragung für normierte Ansetzungen - Einheitstitel', 1, 0, NULL),
		('', '748', 'Verknüpfungseintragung für normierte Ansetzungen - Zeitschlagwort', 'Verknüpfungseintragung für normierte Ansetzungen - Zeitschlagwort', 1, 0, NULL),
		('', '750', 'Verknüpfungseintragung für normierte Ansetzungen - Sachschlagwort', 'Verknüpfungseintragung für normierte Ansetzungen - Sachschlagwort', 1, 0, NULL),
		('', '751', 'Verknüpfungseintragung für normierte Ansetzungen - Geografischer Name', 'Verknüpfungseintragung für normierte Ansetzungen - Geografischer Name', 1, 0, NULL),
		('', '753', 'INDEX TERM--UNCONTROLLED [CLASSIFICATION FORMAT]', 'INDEX TERM--UNCONTROLLED [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '754', 'INDEX TERM--FACETED TOPICAL TERMS [CLASSIFICATION FORMAT]', 'INDEX TERM--FACETED TOPICAL TERMS [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '755', 'Verknüpfungseintragung für normierte Ansetzungen - Genre/Formschlagwort', 'Verknüpfungseintragung für normierte Ansetzungen - Genre/Formschlagwort', 1, 0, NULL),
		('', '761', 'ADD OR DIVIDE LIKE INSTRUCTIONS [CLASSIFICATION FORMAT]', 'ADD OR DIVIDE LIKE INSTRUCTIONS [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '762', 'TABLE IDENTIFICATION [CLASSIFICATION FORMAT]', 'TABLE IDENTIFICATION [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '763', 'INTERNAL SUBARRANGEMENT OR ADD TABLE ENTRY [CLASSIFICATION FORMAT]', 'INTERNAL SUBARRANGEMENT OR ADD TABLE ENTRY [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '764', 'RULE IDENTIFICATION [CLASSIFICATION FORMAT]', 'RULE IDENTIFICATION [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '765', 'SYNTHESIZED NUMBER COMPONENTS [CLASSIFICATION FORMAT]', 'SYNTHESIZED NUMBER COMPONENTS [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '766', 'SECONDARY TABLE INFORMATION [CLASSIFICATION FORMAT]', 'SECONDARY TABLE INFORMATION [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '768', 'CITATION AND PREFERENCE ORDER INSTRUCTIONS [CLASSIFICATION FORMAT]', 'CITATION AND PREFERENCE ORDER INSTRUCTIONS [CLASSIFICATION FORMAT]', 1, 0, NULL),
		('', '780', 'Verknüpfungseintragung der Unterteilung - Allgemeine Unterteilung', 'Verknüpfungseintragung der Unterteilung - Allgemeine Unterteilung', 1, 0, NULL),
		('', '781', 'Verknüpfungseintragung der Unterteilung - Geografische Unterteilung', 'Verknüpfungseintragung der Unterteilung - Geografische Unterteilung', 1, 0, NULL),
		('', '782', 'Verknüpfungseintragung der Unterteilung - Chronologische Unterteilung', 'Verknüpfungseintragung der Unterteilung - Chronologische Unterteilung', 1, 0, NULL),
		('', '785', 'Verknüpfungseintragung der Unterteilung - Unterteilung nach Form', 'Verknüpfungseintragung der Unterteilung - Unterteilung nach Form', 1, 0, NULL),
		('', '788', 'Komplexe Verknüpfungseintragungsdaten', 'Komplexe Verknüpfungseintragungsdaten', 0, 0, NULL),
		('', '856', 'Elektronische Adresse und Zugriff', 'Elektronische Adresse und Zugriff', 1, 0, NULL),
		('', '880', 'Andersschriftliche Darstellung', 'Andersschriftliche Darstellung', 1, 0, NULL),
		('', '883', 'Maschinell generierte Metadatenprovenienz', 'Maschinell generierte Metadatenprovenienz', 1, 0, NULL),
		('', '884', 'Beschreibung der Konvertierung', 'Beschreibung der Konvertierung', 1, 0, NULL),
		('', '942', 'Koha-interne Verwendung', 'Koha-interne Verwendung', 0, 1, NULL);



INSERT INTO `auth_subfield_structure` (`authtypecode`, `tagfield`, `tagsubfield`, `liblibrarian`, `libopac`, `repeatable`, `mandatory`, `tab`, `authorised_value`, `value_builder`, `seealso`, `isurl`, `hidden`, `linkid`, `kohafield`, `frameworkcode`) VALUES
		('', '000', '@', 'Kontrollfeld mit fester Länge', 'Kontrollfeld mit fester Länge', 0, 1, 0, NULL, 'marc21_leader_authorities.pl', NULL, 0, 0, '', '', ''),
		('', '001', '@', 'Kontrollfeld', 'Kontrollfeld', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', 'auth_header.authid', ''),
		('', '003', '@', 'Kontrollfeld', 'Kontrollfeld', 0, 1, 0, NULL, 'marc21_field_003.pl', NULL, 0, 0, '', '', ''),
		('', '005', '@', 'Kontrollfeld', 'Kontrollfeld', 0, 1, 0, NULL, 'marc21_field_005.pl', NULL, 0, 0, '', '', ''),
		('', '008', '@', 'Kontrollfeld mit fester Länge', 'Kontrollfeld mit fester Länge', 0, 1, 0, NULL, 'marc21_field_008_authorities.pl', NULL, 0, 0, '', '', ''),
		('', '010', 'a', 'LC-Kontrollnummer', 'LC-Kontrollnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '010', 'z', 'Gelöschte/ungültige LC-Kontrollnummer', 'Gelöschte/ungültige LC-Kontrollnummer', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '010', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '014', 'a', 'Kontrollnummer des verwandten bibliografischen Datensatzes', 'Kontrollnummer des verwandten bibliografischen Datensatzes', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '014', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '014', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '016', 'a', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '016', 'z', 'Gelöschte oder ungültige Datensatzkontrollnummer', 'Gelöschte oder ungültige Datensatzkontrollnummer', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '016', '2', 'Quelle', 'Quelle', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '016', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', 'a', 'Internationale Standardbuchnummer', 'Internationale Standardbuchnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', 'c', 'Bezugsbedingungen', 'Bezugsbedingungen', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', 'q', 'Nähere Angaben', 'Nähere Angaben', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', 'z', 'Gelöschte/ungültige ISBN', 'Gelöschte/ungültige ISBN', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'a', 'Internationale Standardseriennummer', 'Internationale Standardseriennummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'l', 'ISSN-L', 'ISSN-L', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'm', 'Gelöschte ISSN-L', 'Gelöschte ISSN-L', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'y', 'Falsche ISSN', 'Falsche ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'z', 'Gelöschte ISSN', 'Gelöschte ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'a', 'Standardnummer oder Code', 'Standardnummer oder Code', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'c', 'Bezugsbedingungen', 'Bezugsbedingungen', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'd', 'Zusätzliche Codes zur Standardnummer oder dem Code', 'Zusätzliche Codes zur Standardnummer oder dem Code', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'q', 'Nähere Angaben', 'Nähere Angaben', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'z', 'Gelöschte(r)/ungültige(r) Standardnummer oder Code', 'Gelöschte(r)/ungültige(r) Standardnummer oder Code', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', '2', 'Quelle der Nummer oder des Codes', 'Quelle der Nummer oder des Codes', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'a', 'Anzahl der Werke', 'Anzahl der Werke', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'b', 'Anzahl der Sätze', 'Anzahl der Sätze', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'c', 'Anzahl der Auszüge', 'Anzahl der Auszüge', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'd', 'Titel oder Ansetzung', 'Titel oder Ansetzung', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'e', 'Rolle', 'Rolle', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'g', 'Notenschlüssel', 'Notenschlüssel', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'm', 'Stimme/Instrument', 'Stimme/Instrument', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'n', 'Tonartvorzeichen', 'Tonartvorzeichen', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'o', 'Taktvorzeichen', 'Taktvorzeichen', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'p', 'Musikalische Notierung', 'Musikalische Notierung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'q', 'Allgemeine Fußnote', 'Allgemeine Fußnote', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'r', 'Tonart', 'Tonart', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 's', 'Kodierte Fußnote zur Validität', 'Kodierte Fußnote zur Validität', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 't', 'Text-Incipit', 'Text-Incipit', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'u', 'URL', 'URL', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'y', 'Verknüpfungstext', 'Verknüpfungstext', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'z', 'OPAC-Notiz', 'OPAC-Notiz', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', '2', 'Systemcode', 'Systemcode', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'd', 'Koordinaten - westlichster Längengrad', 'Koordinaten - westlichster Längengrad', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'e', 'Koordinaten - östlichster Längengrad', 'Koordinaten - östlichster Längengrad', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'f', 'Koordinaten - nördlichster Breitengrad', 'Koordinaten - nördlichster Breitengrad', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'g', 'Koordinaten - südlichster Breitengrad', 'Koordinaten - südlichster Breitengrad', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'j', 'Deklination - nördliche Grenze', 'Deklination - nördliche Grenze', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'k', 'Deklination - südliche Grenze', 'Deklination - südliche Grenze', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'm', 'Rektaszension - östliche Grenze', 'Rektaszension - östliche Grenze', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'n', 'Rektaszension - westliche Grenze', 'Rektaszension - westliche Grenze', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'p', 'Äquinoktium', 'Äquinoktium', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'r', 'Entfernung von der Erde', 'Entfernung von der Erde', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 's', 'G-Ring Breitengrad', 'G-Ring Breitengrad', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 't', 'G-Ring Längengrad', 'G-Ring Längengrad', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'x', 'Anfangsdatum', 'Anfangsdatum', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'y', 'Enddatum', 'Enddatum', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'z', 'Name des außerirdischen Himmelskörpers', 'Name des außerirdischen Himmelskörpers', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '2', 'Quelle', 'Quelle', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '3', 'Spezifische Materialangaben', 'Spezifische Materialangaben', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '035', 'a', 'System-Kontrollnummer', 'System-Kontrollnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '035', 'z', 'Gelöschte/ungültige Kontrollnummer', 'Gelöschte/ungültige Kontrollnummer', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '035', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '035', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'a', 'Original-Katalogisierungsstelle', 'Original-Katalogisierungsstelle', 0, 1, 0, NULL, 'marc21_field_003.pl', NULL, 0, 0, '', '', ''),
		('', '040', 'b', 'Katalogisierungssprache', 'Katalogisierungssprache', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'c', 'Übertragungsstelle', 'Übertragungsstelle', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'd', 'Bearbeitungsstelle', 'Bearbeitungsstelle', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'e', 'Beschreibungsfestlegungen', 'Beschreibungsfestlegungen', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'f', 'Schlagwort- oder Thesaurusfestlegungen', 'Schlagwort- oder Thesaurusfestlegungen', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '042', 'a', 'Authentifizierungscode', 'Authentifizierungscode', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', 'a', 'Code für geografische Gebiete', 'Code für geografische Gebiete', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', 'b', 'Lokaler GAC-Code', 'Lokaler GAC-Code', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', 'c', 'ISO-Code', 'ISO-Code', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', '2', 'Quelle des lokalen Codes', 'Quelle des lokalen Codes', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', 'a', 'Zeitabschnittscode', 'Zeitabschnittscode', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', 'b', 'Formatierter Zeitabschnitt 9999 v. Chr. bis n. Chr.', 'Formatierter Zeitabschnitt 9999 v. Chr. bis n. Chr.', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', 'c', 'Formatierter Zeitabschnitt vor 9999 v. Chr.', 'Formatierter Zeitabschnitt vor 9999 v. Chr.', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'f', 'Geburtsdatum', 'Geburtsdatum', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'g', 'Sterbedatum', 'Sterbedatum', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'k', 'Gebildetes Beginn- oder Einzeldatum', 'Gebildetes Beginn- oder Einzeldatum', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'l', 'Gebildetes Enddatum', 'Gebildetes Enddatum', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'o', 'Einzeldatum oder Startdatum des aggregierten Inhalts', 'Einzeldatum oder Startdatum des aggregierten Inhalts', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'p', 'Enddatum für aggregierte Inhalte', 'Enddatum für aggregierte Inhalte', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'q', 'Gründungsdatum', 'Gründungsdatum', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'r', 'Auflösungsdatum', 'Auflösungsdatum', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 's', 'Startperiode', 'Startperiode', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 't', 'Endperiode', 'Endperiode', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'u', 'URL', 'URL', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', '2', 'Quelle der Datumsangaben', 'Quelle der Datumsangaben', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', 'a', 'Notation', 'Notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', 'a', 'Geografischer Klassifikationscode eines Gebiets', 'Geografischer Klassifikationscode eines Gebiets', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', 'b', 'Geografischer Klassifikationscode eines Teilgebiets', 'Geografischer Klassifikationscode eines Teilgebiets', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', 'd', 'Eingetragener Ortsname', 'Eingetragener Ortsname', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', '2', 'Quelle des Codes', 'Quelle des Codes', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', 'a', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', 'b', 'Notationselement - Endnummer des Bereiches', 'Notationselement - Endnummer des Bereiches', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', 'a', 'Notation', 'Notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', 'a', 'LC-Notation - Einzelnummer oder Beginnnummer des Bereiches', 'LC-Notation - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', 'b', 'LC-Notation - Endnummer des Bereiches', 'LC-Notation - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', '5', 'Bibliothek, auf die sich das Feld bezieht', 'Bibliothek, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', 'a', 'Notation', 'Notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '063', 'a', 'NLM-Notation - Einzelnummer oder Beginnnummer des Bereiches', 'NLM-Notation - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '063', 'b', 'NLM-Notation - Endnummer des Bereiches', 'NLM-Notation - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '063', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', 'a', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', 'b', 'Notationselement - Endnummer des Bereiches', 'Notationselement - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', '2', 'Quelle der Nummer', 'Quelle der Nummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '066', 'a', 'Primärer G0-Zeichensatz', 'Primärer G0-Zeichensatz', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '066', 'b', 'Primärer G1-Zeichensatz', 'Primärer G1-Zeichensatz', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '066', 'c', 'Wechselnder G0- oder G1-Zeichensatz', 'Wechselnder G0- oder G1-Zeichensatz', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '068', 'a', 'NLM-Notation - Einzelnummer oder Beginnnummer des Bereiches', 'NLM-Notation - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '068', 'b', 'NLM-Notation - Endnummer des Bereiches', 'NLM-Notation - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '068', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '068', '5', 'Bibliothek, auf die sich das Feld bezieht', 'Bibliothek, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', 'a', 'Notation', 'Notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', 'a', 'Code der Sacherschließungskategorie', 'Code der Sacherschließungskategorie', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', 'x', 'Code der Sacherschließungskategorie, Unterabteilung', 'Code der Sacherschließungskategorie, Unterabteilung', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', '2', 'Quelle des Codes', 'Quelle des Codes', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '073', 'a', 'Verwendung der Unterteilung', 'Verwendung der Unterteilung', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '073', 'z', 'Quelle des Codes', 'Quelle des Codes', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '073', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '073', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', 'a', 'Notation nach der Universal Decimal Classification', 'Notation nach der Universal Decimal Classification', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', 'x', 'Allgemeine Anhängezahlen', 'Allgemeine Anhängezahlen', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', '2', 'Ausgabe-Identifier', 'Ausgabe-Identifier', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'y', 'Laufnummer der internen Anhängetafel', 'Laufnummer der internen Anhängetafel', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', 'a', 'Notation', 'Notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'a', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'b', 'Notationselement - Endnummer des Bereiches', 'Notationselement - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'z', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'a', 'Classification scheme code', 'Classification scheme code', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'b', 'Edition title', 'Edition title', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'c', 'Ausgabe-Identifier', 'Ausgabe-Identifier', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'd', 'Source edition identifier', 'Source edition identifier', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'e', 'Language code', 'Language code', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'f', 'Autorisierende Stelle', 'Autorisierende Stelle', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'n', 'Variations', 'Variations', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', 'a', 'Signatur', 'Signatur', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', 'z', 'Gelöschte/ungültige Signatur', 'Gelöschte/ungültige Signatur', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', '2', 'Quelle der Nummer', 'Quelle der Nummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', 'a', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', 'b', 'Notationselement - Endnummer des Bereiches', 'Notationselement - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', '2', 'Quelle der Nummer', 'Quelle der Nummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '088', 'a', 'Document shelving number (CODOC)', 'Document shelving number (CODOC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'z', 'Gelöschte/ungültige Signatur', 'Gelöschte/ungültige Signatur', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', '2', 'Quelle der Nummer', 'Quelle der Nummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', 'a', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', 'b', 'Notationselement - Endnummer des Bereiches', 'Notationselement - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', 'z', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'a', 'Notation', 'Notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', 'a', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', 'b', 'Notationselement - Endnummer des Bereiches', 'Notationselement - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', 'z', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'a', 'Notation', 'Notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', 'a', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 'Notationselement - Einzelnummer oder Beginnnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', 'b', 'Notationselement - Endnummer des Bereiches', 'Notationselement - Endnummer des Bereiches', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', 'c', 'Erläuternder Term', 'Erläuternder Term', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', 'z', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'a', 'Notation', 'Notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'b', 'Exemplarnummer', 'Exemplarnummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'd', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 'Bände/Datumsangaben, auf die sich die Signatur bezieht', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', '2', 'Ausgabenummer', 'Ausgabenummer', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'a', 'Personenname', 'Personenname', 0, 0, 1, NULL, NULL, '''100b'',''100c'',''100q'',''100d'',''100t'',''100o'',''100m'',''100r'',''100s'',''100k'',''100n'',''100p'',''100g'',''100l'',''100f'',''100h'',''100x'',''100z'',''100y'',''100v''', 0, 0, '', '', ''),
		('', '100', 'b', 'Zählung', 'Zählung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'c', 'Titel und andere Wörter in Verbindung mit einem Namen', 'Titel und andere Wörter in Verbindung mit einem Namen', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'd', 'Datumsangaben in Verbindung mit einem Namen', 'Datumsangaben in Verbindung mit einem Namen', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'e', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'j', 'Zuordnungsvermerk', 'Zuordnungsvermerk', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'n', 'Zählung des Teils/der Abteilung eines Werkes', 'Zählung des Teils/der Abteilung eines Werkes', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'q', 'Vollständige Form eines Namens', 'Vollständige Form eines Namens', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'r', 'Tonart', 'Tonart', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'a', 'Körperschafts- oder Gebietskörperschaftsname als Eintragungselement', 'Körperschafts- oder Gebietskörperschaftsname als Eintragungselement', 0, 0, 1, NULL, NULL, '''110b'',''110c'',''110d'',''110t'',''110o'',''110m'',''110r'',''110s'',''110k'',''110n'',''110p'',''110g'',''110l'',''110f'',''110h'',''110x'',''110z'',''110y'',''110v''', 0, 0, '', '', ''),
		('', '110', 'b', 'Untergeordnete Körperschaft', 'Untergeordnete Körperschaft', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'c', 'Ort des Kongresses', 'Ort des Kongresses', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'd', 'Datum des Kongresses oder der Vertragsunterzeichnung', 'Datum des Kongresses oder der Vertragsunterzeichnung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'e', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'n', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'r', 'Tonart', 'Tonart', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'a', 'Kongress- oder Gebietskörperschaftsname als Eintragungselement', 'Kongress- oder Gebietskörperschaftsname als Eintragungselement', 0, 0, 1, NULL, NULL, '''111e'',''111c'',''111d'',''111t'',''111s'',''111k'',''111n'',''111p'',''111g'',''111l'',''111f'',''111h'',''111x'',''111z'',''111y'',''111v''', 0, 0, '', '', ''),
		('', '111', 'b', 'Nummer [OBSOLETE]', 'Nummer [OBSOLETE]', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'c', 'Ort des Kongresses', 'Ort des Kongresses', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'd', 'Datum des Kongresses', 'Datum des Kongresses', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'e', 'Untergeordnete Körperschaft', 'Untergeordnete Körperschaft', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'j', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'n', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'q', 'Name des Kongresses, der dem Gebietskörperschaftsnamen als Eintragungselement folgt', 'Name des Kongresses, der dem Gebietskörperschaftsnamen als Eintragungselement folgt', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'a', 'Einheitstitel', 'Einheitstitel', 0, 0, 1, NULL, NULL, '''130o'',''130m'',''130r'',''130s'',''130d'',''130k'',''130n'',''130p'',''130g'',''130l'',''130f'',''130h'',''130t'',''130x'',''130z'',''130y'',''130v''', 0, 0, '', '', ''),
		('', '130', 'd', 'Datum der Vertragsunterzeichnung', 'Datum der Vertragsunterzeichnung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'n', 'Zählung des Teils/der Abteilung eines Werkes', 'Zählung des Teils/der Abteilung eines Werkes', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'r', 'Tonart', 'Tonart', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'a', 'Einheitstitel', 'Einheitstitel', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'd', 'Date of treaty', 'Date of treaty', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'h', 'General material designation', 'General material designation', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'l', 'Language', 'Language', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'n', 'Number of part or section/serial, thematic, or opus number', 'Number of part or section/serial, thematic, or opus number', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'o', 'Arranged or arr. for music', 'Arranged or arr. for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'p', 'Part or section', 'Part or section', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'r', 'Tonart', 'Tonart', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'x', 'General subject subdivision', 'General subject subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'y', 'Period subject subdivision', 'Period subject subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'z', 'Place subject subdivision', 'Place subject subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'a', 'Collective title', 'Collective title', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'd', 'Date of treaty', 'Date of treaty', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'h', 'General material designation', 'General material designation', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'l', 'Language', 'Language', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'n', 'Number of part or section/serial, thematic, or opus number', 'Number of part or section/serial, thematic, or opus number', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'o', 'Arranged or arr. for music', 'Arranged or arr. for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'p', 'Part or section', 'Part or section', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'r', 'Tonart', 'Tonart', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 's', 'Version', 'Version', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', 'a', 'Zeitschlagwort', 'Zeitschlagwort', 0, 0, 1, NULL, NULL, '''148y'',''148x'',''148z'',''148v''', 0, 0, '', '', ''),
		('', '148', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'a', 'Sachschlagwort oder geografischer Name als Eintragungselement', 'Sachschlagwort oder geografischer Name als Eintragungselement', 0, 0, 1, NULL, NULL, '''150x'',''150z'',''150y'',''150v''', 0, 0, '', '', ''),
		('', '150', 'b', 'Einem geografischen Namen folgendes Sachschlagwort als Eintragungselement', 'Einem geografischen Namen folgendes Sachschlagwort als Eintragungselement', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'a', 'Geografischer Name', 'Geografischer Name', 0, 0, 1, NULL, NULL, '''151z'',''151x'',''151y'',''151v''', 0, 0, '', '', ''),
		('', '151', 'b', 'Name following place as an entry element {OBSOLETE]', 'Name following place as an entry element {OBSOLETE]', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'j', 'Caption', 'Caption', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'z', 'Table identification', 'Table identification', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', 'a', 'General explanatory index term', 'General explanatory index term', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', 'b', 'General explanatory index term--Succeeding level', 'General explanatory index term--Succeeding level', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', 'f', 'Schedule identification', 'Schedule identification', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', 'a', 'Genre/Formschlagwort', 'Genre/Formschlagwort', 0, 0, 1, NULL, NULL, '''155v'',''155x'',''155z'',''155y''', 0, 0, '', '', ''),
		('', '155', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '162', 'a', 'Bezeichnung der Besetzung', 'Bezeichnung der Besetzung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '162', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '162', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'a', 'Classification number referred to--Single number or beginning number of span', 'Classification number referred to--Single number or beginning number of span', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'c', 'Classification number referred to--Ending number of span', 'Classification number referred to--Ending number of span', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'z', 'Table identification', 'Table identification', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', 'a', 'Ansetzung verweist auf', 'Ansetzung verweist auf', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', '0', 'IDN des Normdatensatzes', 'IDN des Normdatensatzes', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', 'a', 'Inhaltstypterm', 'Inhaltstypterm', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', 'b', 'Inhaltstypcode', 'Inhaltstypcode', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '2', 'Quelle', 'Quelle', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '3', 'Spezifische Materialangaben', 'Spezifische Materialangaben', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', 'a', 'Begriff für das Notenformat', 'Begriff für das Notenformat', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', 'b', 'Code für das Notenformat', 'Code für das Notenformat', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '3', 'Spezifische Materialangaben', 'Spezifische Materialangaben', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'a', 'Classification number referred to--Single number or beginning number of span', 'Classification number referred to--Single number or beginning number of span', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'c', 'Classification number referred to--Ending number of span', 'Classification number referred to--Ending number of span', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'z', 'Table identification', 'Table identification', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', 'a', 'Ansetzung verweist auf', 'Ansetzung verweist auf', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', '0', 'IDN des Normdatensatzes', 'IDN des Normdatensatzes', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'a', 'Art der Körperschaft', 'Art der Körperschaft', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'b', 'Art der Zugehörigkeit', 'Art der Zugehörigkeit', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'c', 'Andere Bezeichnung', 'Andere Bezeichnung', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'd', 'Titel der Person', 'Titel der Person', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 's', 'Startperiode', 'Startperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 't', 'Endperiode', 'Endperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', '2', 'Quelle', 'Quelle', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'a', 'Geburtsort', 'Geburtsort', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'b', 'Sterbeort', 'Sterbeort', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'c', 'Zugehöriges Land', 'Zugehöriges Land', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'e', 'Wohnort/Ort des Hauptsitzes', 'Wohnort/Ort des Hauptsitzes', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'f', 'Anderer zugehöriger Ort', 'Anderer zugehöriger Ort', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'g', 'Ursprungsort des Werks', 'Ursprungsort des Werks', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 's', 'Startperiode', 'Startperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 't', 'Endperiode', 'Endperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'a', 'Adresse', 'Adresse', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'b', 'Ort', 'Ort', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'c', 'Zwischenzugehörigkeit', 'Zwischenzugehörigkeit', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'd', 'Land', 'Land', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'e', 'Postleitzahl', 'Postleitzahl', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'm', 'E-Mail-Adresse', 'E-Mail-Adresse', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 's', 'Startperiode', 'Startperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 't', 'Endperiode', 'Endperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'z', 'OPAC-Notiz', 'OPAC-Notiz', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', '4', 'Funktionsbezeichnungscode', 'Funktionsbezeichnungscode', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 'a', 'Tätigkeitsbereich', 'Tätigkeitsbereich', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 's', 'Startperiode', 'Startperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 't', 'Endperiode', 'Endperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 'a', 'Zugehörige Gruppe', 'Zugehörige Gruppe', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 's', 'Startperiode', 'Startperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 't', 'Endperiode', 'Endperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 'a', 'Beruf', 'Beruf', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 's', 'Startperiode', 'Startperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 't', 'Endperiode', 'Endperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 'a', 'Geschlecht', 'Geschlecht', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 's', 'Startperiode', 'Startperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 't', 'Endperiode', 'Endperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'a', 'Art der Familie', 'Art der Familie', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'b', 'Berühmtes Familienmitglied', 'Berühmtes Familienmitglied', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'c', 'Erbtitel', 'Erbtitel', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 's', 'Startperiode', 'Startperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 't', 'Endperiode', 'Endperiode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', 'a', 'Language code', 'Language code', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', 'l', 'Sprachbezeichnung', 'Sprachbezeichnung', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', '2', 'Quelle des Codes', 'Quelle des Codes', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', 'q', 'Vollständigere Namensform', 'Vollständigere Namensform', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', 'a', 'Form des Werks', 'Form des Werks', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', 'a', 'Andere unterscheidende Eigenschaft', 'Andere unterscheidende Eigenschaft', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', 'u', 'URL', 'URL', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', 'v', 'Informationsquelle', 'Informationsquelle', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'a', 'Besetzung', 'Besetzung', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'b', 'Solist', 'Solist', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'd', 'Doppelt besetztes Instrument', 'Doppelt besetztes Instrument', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'e', 'Anzahl der Ensembles', 'Anzahl der Ensembles', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'n', 'Anzahl der Künstler eines Mediums', 'Anzahl der Künstler eines Mediums', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'p', 'Alternative Besetzung', 'Alternative Besetzung', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 's', 'Gesamtzahl der Künstler', 'Gesamtzahl der Künstler', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'v', 'Fußnote', 'Fußnote', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'a', 'Seriennummer', 'Seriennummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'b', 'Opusnummer', 'Opusnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'c', 'Nummer des thematischen Index', 'Nummer des thematischen Index', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'd', 'Code des thematischen Index', 'Code des thematischen Index', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'e', 'Verleger, der mit der Opusnummer in Verbindung steht', 'Verleger, der mit der Opusnummer in Verbindung steht', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', '2', 'Quelle', 'Quelle', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '384', 'a', 'Tonart', 'Tonart', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '384', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '384', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', 'a', 'Zielgruppenbegriff', 'Zielgruppenbegriff', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', 'b', 'Zielgruppencode', 'Zielgruppencode', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', 'm', 'Demographischer Gruppenbegriff', 'Demographischer Gruppenbegriff', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', 'n', 'Demographischer Gruppencode', 'Demographischer Gruppencode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '2', 'Quelle', 'Quelle', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '3', 'Spezifische Materialangaben', 'Spezifische Materialangaben', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', 'a', 'Bezeichnung für Urheber/Mitwirkenden', 'Bezeichnung für Urheber/Mitwirkenden', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', 'b', 'Code für Mitwirkenden', 'Code für Mitwirkenden', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', 'm', 'Demographischer Gruppenbegriff', 'Demographischer Gruppenbegriff', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', 'n', 'Demographischer Gruppencode', 'Demographischer Gruppencode', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '2', 'Quelle', 'Quelle', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '3', 'Spezifische Materialangaben', 'Spezifische Materialangaben', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '3', 'Spezifische Materialangaben', 'Spezifische Materialangaben', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', 'a', 'Begriff für den Zeitabschnitt der Erstellung', 'Begriff für den Zeitabschnitt der Erstellung', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'a', 'Personenname', 'Personenname', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, '', '', 'PERSO_NAME'),
		('', '400', 'b', 'Zählung', 'Zählung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'c', 'Titel und andere Wörter in Verbindung mit einem Namen', 'Titel und andere Wörter in Verbindung mit einem Namen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'd', 'Datumsangaben in Verbindung mit einem Namen', 'Datumsangaben in Verbindung mit einem Namen', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'e', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'j', 'Zuordnungsvermerk', 'Zuordnungsvermerk', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'n', 'Zählung des Teils/der Abteilung eines Werkes', 'Zählung des Teils/der Abteilung eines Werkes', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'q', 'Vollständige Form eines Namens', 'Vollständige Form eines Namens', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'r', 'Tonart', 'Tonart', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '410', 'a', 'Körperschafts- oder Gebietskörperschaftsname als Eintragungselement', 'Körperschafts- oder Gebietskörperschaftsname als Eintragungselement', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, '', '', 'CORPO_NAME'),
		('', '410', 'b', 'Untergeordnete Körperschaft', 'Untergeordnete Körperschaft', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'c', 'Ort des Kongresses', 'Ort des Kongresses', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'd', 'Datum des Kongresses oder der Vertragsunterzeichnung', 'Datum des Kongresses oder der Vertragsunterzeichnung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'e', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'n', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'r', 'Tonart', 'Tonart', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '411', 'a', 'Kongress- oder Gebietskörperschaftsname als Eintragungselement', 'Kongress- oder Gebietskörperschaftsname als Eintragungselement', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, '', '', 'MEETI_NAME'),
		('', '411', 'b', 'Nummer {OBSOLETE]', 'Nummer {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'c', 'Ort des Kongresses', 'Ort des Kongresses', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'd', 'Datum des Kongresses', 'Datum des Kongresses', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'e', 'Untergeordnete Körperschaft', 'Untergeordnete Körperschaft', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'j', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'n', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'q', 'Name des Kongresses, der dem Gebietskörperschaftsnamen als Eintragungselement folgt', 'Name des Kongresses, der dem Gebietskörperschaftsnamen als Eintragungselement folgt', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '430', 'a', 'Einheitstitel', 'Einheitstitel', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, '', '', 'UNIF_TITLE'),
		('', '430', 'd', 'Datum der Vertragsunterzeichnung', 'Datum der Vertragsunterzeichnung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'n', 'Zählung des Teils/der Abteilung eines Werkes', 'Zählung des Teils/der Abteilung eines Werkes', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'r', 'Tonart', 'Tonart', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '448', 'a', 'Zeitschlagwort', 'Zeitschlagwort', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, '', '', 'CHRON_TERM'),
		('', '448', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '450', 'a', 'Sachschlagwort oder geografischer Name als Eintragungselement', 'Sachschlagwort oder geografischer Name als Eintragungselement', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, '', '', 'TOPIC_TERM'),
		('', '450', 'b', 'Einem geografischen Namen folgendes Sachschlagwort als Eintragungselement', 'Einem geografischen Namen folgendes Sachschlagwort als Eintragungselement', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '451', 'a', 'Geografischer Name', 'Geografischer Name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, '', '', 'GEOGR_NAME'),
		('', '451', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '453', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'c', 'Classification element--Ending number of span', 'Classification element--Ending number of span', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'j', 'Caption', 'Caption', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 't', 'Topic', 'Topic', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'z', 'Table identification', 'Table identification', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '455', 'a', 'Genre/Formschlagwort', 'Genre/Formschlagwort', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, '', '', 'GENRE/FORM'),
		('', '455', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '462', 'a', 'Bezeichnung der Besetzung', 'Bezeichnung der Besetzung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'a', 'Personenname', 'Personenname', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, '', '', 'PERSO_NAME'),
		('', '500', 'b', 'Zählung', 'Zählung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'c', 'Titel und andere Wörter in Verbindung mit einem Namen', 'Titel und andere Wörter in Verbindung mit einem Namen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'd', 'Datumsangaben in Verbindung mit einem Namen', 'Datumsangaben in Verbindung mit einem Namen', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'e', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'j', 'Zuordnungsvermerk', 'Zuordnungsvermerk', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'n', 'Zählung des Teils/der Abteilung eines Werkes', 'Zählung des Teils/der Abteilung eines Werkes', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'q', 'Vollständige Form eines Namens', 'Vollständige Form eines Namens', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'r', 'Tonart', 'Tonart', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '510', 'a', 'Körperschafts- oder Gebietskörperschaftsname als Eintragungselement', 'Körperschafts- oder Gebietskörperschaftsname als Eintragungselement', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, '', '', 'CORPO_NAME'),
		('', '510', 'b', 'Untergeordnete Körperschaft', 'Untergeordnete Körperschaft', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'c', 'Ort des Kongresses', 'Ort des Kongresses', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'd', 'Datum des Kongresses oder der Vertragsunterzeichnung', 'Datum des Kongresses oder der Vertragsunterzeichnung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'e', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'n', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'r', 'Tonart', 'Tonart', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '511', 'a', 'Kongress- oder Gebietskörperschaftsname als Eintragungselement', 'Kongress- oder Gebietskörperschaftsname als Eintragungselement', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, '', '', 'MEETI_NAME'),
		('', '511', 'c', 'Ort des Kongresses', 'Ort des Kongresses', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'd', 'Datum des Kongresses', 'Datum des Kongresses', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'e', 'Untergeordnete Körperschaft', 'Untergeordnete Körperschaft', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'j', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'n', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'q', 'Name des Kongresses, der dem Gebietskörperschaftsnamen als Eintragungselement folgt', 'Name des Kongresses, der dem Gebietskörperschaftsnamen als Eintragungselement folgt', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '530', 'a', 'Einheitstitel', 'Einheitstitel', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, '', '', 'UNIF_TITLE'),
		('', '530', 'd', 'Datum der Vertragsunterzeichnung', 'Datum der Vertragsunterzeichnung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'n', 'Zählung des Teils/der Abteilung eines Werkes', 'Zählung des Teils/der Abteilung eines Werkes', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'r', 'Tonart', 'Tonart', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '548', 'a', 'Zeitschlagwort', 'Zeitschlagwort', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, '', '', 'CHRON_TERM'),
		('', '548', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '550', 'a', 'Sachschlagwort oder geografischer Name als Eintragungselement', 'Sachschlagwort oder geografischer Name als Eintragungselement', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, '', '', 'TOPIC_TERM'),
		('', '550', 'b', 'Einem geografischen Namen folgendes Sachschlagwort als Eintragungselement', 'Einem geografischen Namen folgendes Sachschlagwort als Eintragungselement', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '551', 'a', 'Geografischer Name', 'Geografischer Name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, '', '', 'GEOGR_NAME'),
		('', '551', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '553', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'j', 'Caption', 'Caption', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 't', 'Topic', 'Topic', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'z', 'Table identification', 'Table identification', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '555', 'a', 'Genre/Formschlagwort', 'Genre/Formschlagwort', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, '', '', 'GENRE/FORM'),
		('', '555', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '9', 'Koha-Normdatenidentnummer', 'Koha-Normdatenidentnummer', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '562', 'a', 'Bezeichnung der Besetzung', 'Bezeichnung der Besetzung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '640', 'a', 'Datumsangaben der Veröffentlichung und/oder Bandbezeichnung', 'Datumsangaben der Veröffentlichung und/oder Bandbezeichnung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '640', 'z', 'Informationsquelle', 'Informationsquelle', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '640', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '640', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '641', 'a', 'Fußnote zu Besonderheiten der Zählung', 'Fußnote zu Besonderheiten der Zählung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '641', 'z', 'Informationsquelle', 'Informationsquelle', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '641', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '641', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', 'a', 'Beispiel für die Zählung von Gesamttiteln', 'Beispiel für die Zählung von Gesamttiteln', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', 'd', 'Band-/Datumsangaben, auf die sich das Beispiel für die Zählung von Gesamttiteln bezieht', 'Band-/Datumsangaben, auf die sich das Beispiel für die Zählung von Gesamttiteln bezieht', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', '5', 'Institution/Kopie, auf die sich das Feld bezieht', 'Institution/Kopie, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', 'a', 'Ort', 'Ort', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', 'b', 'Herausgebende Körperschaft', 'Herausgebende Körperschaft', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', 'd', 'Band-/Datumsangaben, auf die sich Ort und herausgebende Körperschaft bezieht', 'Band-/Datumsangaben, auf die sich Ort und herausgebende Körperschaft bezieht', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', 'a', 'Analyse-Praxis von Gesamttiteln', 'Analyse-Praxis von Gesamttiteln', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', 'b', 'Ausnahmen von der Analyse-Praxis', 'Ausnahmen von der Analyse-Praxis', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', 'd', 'Band-/Datumsangaben auf die sich die Analyse-Praxis beziehen', 'Band-/Datumsangaben auf die sich die Analyse-Praxis beziehen', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', '5', 'Institution/Kopie, auf die sich das Feld bezieht', 'Institution/Kopie, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', 'a', 'Verweisungspraxis von Gesamttiteln', 'Verweisungspraxis von Gesamttiteln', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', 'd', 'Band-/Datumsangaben, auf die sich die Verweisungs-Praxis beziehen', 'Band-/Datumsangaben, auf die sich die Verweisungs-Praxis beziehen', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', '5', 'Institution/Kopie, auf die sich das Feld bezieht', 'Institution/Kopie, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', 'a', 'Klassifizierungspraxis von Gesamttiteln', 'Klassifizierungspraxis von Gesamttiteln', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', 'd', 'Band-/Datumsangaben, auf die sich die Klassifizierungspraxis beziehen', 'Band-/Datumsangaben, auf die sich die Klassifizierungspraxis beziehen', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', 'a', 'Erläuterung', 'Erläuterung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', 'b', 'Ansetzung verweist auf', 'Ansetzung verweist auf', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', 't', 'Titel verweist auf', 'Titel verweist auf', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', 'a', 'Erläuterung', 'Erläuterung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', 'b', 'Ansetzung verweist auf', 'Ansetzung verweist auf', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', 't', 'Titel verweist auf', 'Titel verweist auf', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '665', 'a', 'Verlauf der Verweisung', 'Verlauf der Verweisung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '665', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '665', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '666', 'a', 'Allgemeine Hinweiserläuterung', 'Allgemeine Hinweiserläuterung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '666', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '666', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '667', 'a', 'Interne allgemeine Fußnote', 'Interne allgemeine Fußnote', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '667', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '667', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '667', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '668', 'a', 'Characters in nonroman alphabet', 'Characters in nonroman alphabet', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', 'a', 'Quellenangabe', 'Quellenangabe', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', 'b', 'Vorhandene Informationen', 'Vorhandene Informationen', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', 'u', 'URL', 'URL', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', 'w', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '671', 'a', 'Citation', 'Citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '671', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', 'a', 'Titel', 'Titel', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', 'b', 'Zusatz zum Titel', 'Zusatz zum Titel', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', 'f', 'Datum', 'Datum', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', 'w', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', 'a', 'Titel', 'Titel', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', 'b', 'Zusatz zum Titel', 'Zusatz zum Titel', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', 'f', 'Datum', 'Datum', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', 'w', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '675', 'a', 'Quellenangabe', 'Quellenangabe', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '675', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '675', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '676', 'a', 'Rule number(s) and additional information', 'Rule number(s) and additional information', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '676', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', 'a', 'Fußnote zu biografischen oder historischen Daten', 'Fußnote zu biografischen oder historischen Daten', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', 'b', 'Erweiterung', 'Erweiterung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', 'u', 'URL', 'URL', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', 'a', 'Ansetzung oder Unterteilung', 'Ansetzung oder Unterteilung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', 'c', 'Classification number--Ending number of span [CLASSIFICATION FORMAT]', 'Classification number--Ending number of span [CLASSIFICATION FORMAT]', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', 'a', 'Schlagwort- oder Unterteilungsansetzung', 'Schlagwort- oder Unterteilungsansetzung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', 'c', 'Classification number--Ending number of span [CLASSIFICATION FORMAT]', 'Classification number--Ending number of span [CLASSIFICATION FORMAT]', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', 'a', 'Zu ersetzende Ansetzung', 'Zu ersetzende Ansetzung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', '0', 'Kontrollnummer des Zieldatensatzes', 'Kontrollnummer des Zieldatensatzes', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'j', 'Caption', 'Caption', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '685', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '685', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '686', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '686', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '687', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '687', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '688', 'a', 'Fußnote zur Anwendungsgeschichte', 'Fußnote zur Anwendungsgeschichte', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '688', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '688', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '688', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'a', 'Personenname', 'Personenname', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'b', 'Zählung', 'Zählung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'c', 'Titel und andere Wörter in Verbindung mit einem Namen', 'Titel und andere Wörter in Verbindung mit einem Namen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'd', 'Datumsangaben in Verbindung mit einem Namen', 'Datumsangaben in Verbindung mit einem Namen', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'e', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'j', 'Zuordnungsvermerk', 'Zuordnungsvermerk', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'n', 'Zählung des Teils/der Abteilung eines Werkes', 'Zählung des Teils/der Abteilung eines Werkes', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'q', 'Vollständige Form eines Namens', 'Vollständige Form eines Namens', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'r', 'Tonart', 'Tonart', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '3', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'a', 'Körperschafts- oder Gebietskörperschaftsname als Eintragungselement', 'Körperschafts- oder Gebietskörperschaftsname als Eintragungselement', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'b', 'Untergeordnete Körperschaft', 'Untergeordnete Körperschaft', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'c', 'Ort des Kongresses', 'Ort des Kongresses', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'd', 'Datum des Kongresses oder der Vertragsunterzeichnung', 'Datum des Kongresses oder der Vertragsunterzeichnung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'e', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'n', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'r', 'Tonart', 'Tonart', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '3', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'a', 'Kongress- oder Gebietskörperschaftsname als Eintragungselement', 'Kongress- oder Gebietskörperschaftsname als Eintragungselement', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'c', 'Ort des Kongresses', 'Ort des Kongresses', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'd', 'Datum des Kongresses', 'Datum des Kongresses', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'e', 'Untergeordnete Körperschaft', 'Untergeordnete Körperschaft', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'j', 'Funktionsbezeichnung', 'Funktionsbezeichnung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'n', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 'Zählung eines Teils/einer Abteilung/eines Kongresses', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'q', 'Name des Kongresses, der dem Gebietskörperschaftsnamen als Eintragungselement folgt', 'Name des Kongresses, der dem Gebietskörperschaftsnamen als Eintragungselement folgt', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '3', 'Spezifische Materialangaben', 'Spezifische Materialangaben', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'a', 'Einheitstitel', 'Einheitstitel', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'd', 'Datum der Vertragsunterzeichnung', 'Datum der Vertragsunterzeichnung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'f', 'Erscheinungsjahr eines Werkes', 'Erscheinungsjahr eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'k', 'Formbestandteil der Ansetzung', 'Formbestandteil der Ansetzung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'l', 'Sprache eines Werkes', 'Sprache eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'm', 'Medium der Musikaufführung', 'Medium der Musikaufführung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'n', 'Zählung des Teils/der Abteilung eines Werkes', 'Zählung des Teils/der Abteilung eines Werkes', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'o', 'Angabe des Musikarrangements', 'Angabe des Musikarrangements', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'p', 'Titel eines Teils/einer Abteilung eines Werkes', 'Titel eines Teils/einer Abteilung eines Werkes', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'r', 'Tonart', 'Tonart', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 't', 'Titel eines Werkes', 'Titel eines Werkes', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '3', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'a', 'Zeitschlagwort', 'Zeitschlagwort', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '3', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'a', 'Sachschlagwort oder geografischer Name als Eintragungselement', 'Sachschlagwort oder geografischer Name als Eintragungselement', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'b', 'Einem geografischen Namen folgendes Sachschlagwort als Eintragungselement', 'Einem geografischen Namen folgendes Sachschlagwort als Eintragungselement', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'c', 'Ort des Ereignisses', 'Ort des Ereignisses', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'd', 'Active date', 'Active date', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '3', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'a', 'Geografischer Name', 'Geografischer Name', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'g', 'Sonstige Informationen', 'Sonstige Informationen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '3', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 'Spezifische Materialangaben [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'a', 'Index term', 'Index term', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'b', 'Index term--Succeeding level', 'Index term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'd', 'Index term referred from', 'Index term referred from', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 's', 'See also reference term', 'See also reference term', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 't', 'See also reference term--Succeeding level', 'See also reference term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'u', 'Use reference term', 'Use reference term', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'v', 'Use reference term--Succeeding level', 'Use reference term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'a', 'Fokussierter Term', 'Fokussierter Term', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'b', 'Nicht-fokussierter Term', 'Nicht-fokussierter Term', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'c', 'Benennung der Normierung/Hierarchie', 'Benennung der Normierung/Hierarchie', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', '3', 'Material specified', 'Material specified', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'a', 'Genre/Formschlagwort als Eintragungselement', 'Genre/Formschlagwort als Eintragungselement', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'a', 'Number where instructions are found--Single number or beginnning number of span', 'Number where instructions are found--Single number or beginnning number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'b', 'Base number', 'Base number', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'd', 'Divided like number', 'Divided like number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'f', 'Facet designator', 'Facet designator', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'n', 'Negative example class number', 'Negative example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'r', 'Root number', 'Root number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'x', 'Andere Notation', 'Andere Notation', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'z', 'Table identification', 'Table identification', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', 'a', 'Besetzung als Ansetzung', 'Besetzung als Ansetzung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', 'z', 'Table number', 'Table number', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'b', 'Base number', 'Base number', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'd', 'Divided like number', 'Divided like number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'j', 'Caption', 'Caption', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'm', 'Manual note', 'Manual note', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'n', 'Number where instructions are found', 'Number where instructions are found', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'r', 'Root number', 'Root number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 's', 'See reference', 'See reference', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'x', 'Andere Notation', 'Andere Notation', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'z', 'Table identification', 'Table identification', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '764', 'a', 'Rule number', 'Rule number', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '764', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '764', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '764', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'a', 'Number where instructions are found--Single number or beginning number of span', 'Number where instructions are found--Single number or beginning number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'b', 'Base number', 'Base number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'c', 'Number where instructions are found--Ending number of span', 'Number where instructions are found--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'f', 'Facet designator', 'Facet designator', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'r', 'Root number', 'Root number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 's', 'Digits added from classification number in schedule or external table', 'Digits added from classification number in schedule or external table', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 't', 'Digits added from internal subarrangement or add table', 'Digits added from internal subarrangement or add table', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'u', 'Number being analyzed', 'Number being analyzed', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'v', 'Notation aus der internen Anhängetafel mit Syntheseanweisung', 'Notation aus der internen Anhängetafel mit Syntheseanweisung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'w', 'Table identification--Internal subarrangement or add table', 'Table identification--Internal subarrangement or add table', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'z', 'Table identification', 'Table identification', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '766', 'a', 'Secondary table of applicability', 'Secondary table of applicability', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '766', 'y', 'Type of division', 'Type of division', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '766', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '766', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'j', 'Caption', 'Caption', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'n', 'Negative example class number', 'Negative example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 't', 'Topic used as example of citation and preference order instruction', 'Topic used as example of citation and preference order instruction', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'x', 'Exception to table of preference', 'Exception to table of preference', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'z', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 'Hilfstafelkennzeichnung-Hilfstafelnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'i', 'Verweisungsphrase', 'Verweisungsphrase', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'v', 'Unterteilung nach Form', 'Unterteilung nach Form', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'w', 'Kontrollunterfeld', 'Kontrollunterfeld', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'x', 'Allgemeine Unterteilung', 'Allgemeine Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'y', 'Chronologische Unterteilung', 'Chronologische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'z', 'Geografische Unterteilung', 'Geografische Unterteilung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', 'a', 'Ansetzung verweist auf', 'Ansetzung verweist auf', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', 'i', 'Erläuterung', 'Erläuterung', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '0', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '2', 'Quelle der Ansetzung oder des Terms', 'Quelle der Ansetzung oder des Terms', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '4', 'Beziehungskennzeichen', 'Beziehungskennzeichen', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '5', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 'Das Unterfeld gibt die Institution an, auf die sich das Feld bezieht', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'a', 'Host Name', 'Host Name', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'b', 'Zugriffsnummer', 'Zugriffsnummer', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'c', 'Angaben zur Komprimierung', 'Angaben zur Komprimierung', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'd', 'Pfad', 'Pfad', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'f', 'Elektronischer Name', 'Elektronischer Name', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'h', 'Bearbeiter der Anfrage', 'Bearbeiter der Anfrage', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'i', 'Anweisung', 'Anweisung', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'j', 'Bits pro Sekunde', 'Bits pro Sekunde', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'k', 'Passwort', 'Passwort', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'l', 'Login', 'Login', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'm', 'Kontaktstelle für Hilfe beim Zugriff', 'Kontaktstelle für Hilfe beim Zugriff', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'n', 'Name des Standortes des Hosts', 'Name des Standortes des Hosts', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'o', 'Betriebssystem', 'Betriebssystem', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'q', 'Elektronisches Format', 'Elektronisches Format', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'r', 'Einstellungen', 'Einstellungen', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 's', 'Dateigröße', 'Dateigröße', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 't', 'Terminalemulation', 'Terminalemulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'u', 'URL', 'URL', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'v', 'Zugriffszeiten', 'Zugriffszeiten', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'w', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'x', 'Interne Anmerkungen', 'Interne Anmerkungen', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'y', 'Verknüpfungstext', 'Verknüpfungstext', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'z', 'OPAC-Notiz', 'OPAC-Notiz', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', '2', 'Zugriffsart', 'Zugriffsart', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', '3', 'Spezifische Materialangaben', 'Spezifische Materialangaben', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '6', 'Verknüpfung', 'Verknüpfung', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'a', 'Generierungsprozess', 'Generierungsprozess', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'c', 'Vertrauenswert', 'Vertrauenswert', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'd', 'Erstellungsdatum der Kopie', 'Erstellungsdatum der Kopie', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'q', 'Einrichtung, welche die Kopien erstellt hat', 'Einrichtung, welche die Kopien erstellt hat', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'x', 'Gültigkeitsende', 'Gültigkeitsende', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'u', 'URL', 'URL', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'w', 'Datensatzkontrollnummer', 'Datensatzkontrollnummer', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', '0', 'IDN des Normdatensatzes oder Standardnummer', 'IDN des Normdatensatzes oder Standardnummer', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', '8', 'Feldverknüpfung und Reihenfolge', 'Feldverknüpfung und Reihenfolge', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'a', 'Konvertierungsprozess', 'Konvertierungsprozess', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'g', 'Konvertierungsdatum', 'Konvertierungsdatum', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'k', 'Identifier für die Quelle der Metadaten', 'Identifier für die Quelle der Metadaten', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'q', 'Konvertierungseinrichtung', 'Konvertierungseinrichtung', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'u', 'URL', 'URL', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '942', 'a', 'Koha-Normsatztyp', 'Koha-Normsatztyp', 0, 1, 9, NULL, NULL, NULL, 0, 8, '', 'auth_header.authtypecode', '');


-- ******************************************************

-- *******************************************************************
-- PERSONAL NAME AUTHORITY FIELDS/SUBFIELDS. XX0
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'PERSO_NAME', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ('110','111','130','148','150','151','155','162',
                     '180','181','182','185','034','640','641','642',
                     '643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'PERSO_NAME', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ('110','111','130','148','150','151','155','162',
                     '180','181','182','185','034','640','641','642',
                     '643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- CORPORATE NAME AUTHORITY FIELDS/SUBFIELDS. X10
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'CORPO_NAME', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','111','130','148','150','151','155','162',
                      '180','181','182','185','034','640','641','642',
                      '643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'CORPO_NAME', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','111','130','148','150','151','155','162',
                      '180','181','182','185','034','640','641','642',
                      '643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- MEETING NAME AUTHORITY FIELDS/SUBFIELDS. X11
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'MEETI_NAME', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','130','148','150','151','155','162',
                      '180','181','182','185','640','641','642','643',
                      '644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'MEETI_NAME', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','130','148','150','151','155','162',
                      '180','181','182','185','640','641','642','643',
                      '644','645','646');

-- ******************************************************

-- *******************************************************************
-- UNIFORM TITLE AUTHORITY FIELDS/SUBFIELDS. X30
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'UNIF_TITLE', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','148','150','151','155','162',
                      '180','181','182','185');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'UNIF_TITLE', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','148','150','151','155','162',
                      '180','181','182','185');

-- ******************************************************

-- *******************************************************************
-- CHRONOLOGICAL TERMS AUTHORITY FIELDS/SUBFIELDS. X48
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'CHRON_TERM', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','150','151','155','162',
                      '180','181','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'CHRON_TERM', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','150','151','155','162',
                      '180','181','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- TOPICAL TERMS AUTHORITY FIELDS/SUBFIELDS. X50
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'TOPIC_TERM', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','151','155','162',
                      '181','182','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'TOPIC_TERM', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','151','155','162',
                      '181','182','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- GEOGRAPHIC NAMES AUTHORITY FIELDS/SUBFIELDS. X51
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'GEOGR_NAME', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','150','155','162',
                      '180','182','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'GEOGR_NAME', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','150','155','162',
                      '180','182','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- GENRE/FORM TERMS AUTHORITY FIELDS/SUBFIELDS. X55
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'GENRE/FORM', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','150','151','162',
                      '180','181','182','014','020','022','031','640',
                      '641','642','643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'GENRE/FORM', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','150','151','162',
                      '180','181','182','014','020','022','031','640',
                      '641','642','643','644','645','646');

-- ******************************************************
